%     Ventricular action potential model ORdmJ
%     Copyright (C) 2020 Jordi Llopis. Contact: jorllolo@etsii.upv.es
% 
%     This program is free software: you can redistribute it and/or modify
%     it under the terms of the GNU General Public License as published by
%     the Free Software Foundation, either version 3 of the License, or
%     (at your option) any later version.
% 
%     This program is distributed in the hope that it will be useful,
%     but WITHOUT ANY WARRANTY; without even the implied warranty of
%     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%     GNU General Public License for more details.
% 
%     You should have received a copy of the GNU General Public License
%     along with this program.  If not, see <https://www.gnu.org/licenses/>

function [ TEAD, output] = calculate_TEAD(parameters)
% This function iteratively searches for the minimum concentration that
% triggers an EAD and computed the index TEAD. It is assumed that
% concentration stored in parameters.D is the EFTPC
% INPUTS:
% parameters: a structure array containing numbers of beats of the
% simulation, cycle length, initial conditions, cell type, drug
% concentration...
% OUTPUTS:
% TEAD: value of the index TEAD
% output: a structure array containing time vector, state variable and
% currents correspondig to the last 3 beat simulated under drug
% concentration leading to an EAD
step=10^0.1;
[time, X, IsJs] = runEADprotocol(parameters);
EFTPC=parameters.D;
ind=find(time==3000,1,'last');
t=time(ind:end)-3000; v=X(ind:end,1); % The last beat is selected
% Calculus of the derivative
d(length(v)) = (v(end) - v(end-1) ) / (t(end)-t(end-1));
ndx = 2:(length(v)-1);
d(1)= ( v(2) - v(1) ) / (t(2)-t(1));
d(ndx) = (v( ndx+1) - v(ndx-1))./ (2 * (t(ndx)-t(ndx-1)));
% Repolarization abnormalites
% - positive derivative 150 ms after the AP peak
% - Vm did not reach the resting condition following an AP upstroke
[Vp, Vp_index]=max(v);
ind=find(t>=(t(Vp_index)+150),1);
d(1:ind)=[];
EADs = find(d>0.01);
[Vr, Vr_index]=min(v(ind:end));
[Vp, Vp_index]=max(v);
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
if ~isempty(EADs) || Vr > -40 || Vp < 0 % EADs at EFTPC
    red=1;
    disp('Concentration should be reduced')
else
    red=0; % No presence of EADs. [] should be increase
    disp('Concentration should be increased')
end
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
D=parameters.D; i=0; it=0;
while (isempty(EADs) && Vr < -40) && it<60 && Vp > 0
    disp('Increasing concentration')
    D=D*step;
    parameters.D=D;
    [time, X, IsJs] = runEADprotocol(parameters);
    ind=find(time==3000,1,'last');
    t=time(ind:end)-3000; v=X(ind:end,1);
    d(length(v)) = (v(end) - v(end-1) ) / (t(end)-t(end-1));
    ndx = 2:(length(v)-1);
    d(1)= ( v(2) - v(1) ) / (t(2)-t(1));
    d(ndx) = (v( ndx+1) - v(ndx-1))./ (2 * (t(ndx)-t(ndx-1)));
    [Vp, Vp_index]=max(v);
    ind=find(t>=(t(Vp_index)+150),1);
    d(1:ind)=[];
    EADs = find(d>0.01);
    [Vr, Vr_index]=min(v(ind:end));
    [Vp, Vp_index]=max(v);
    i=i+1;
    disp(['Increasing []. ' num2str(i) ' times'])
    it=it+1;
end
DEAD=D;
TEAD=DEAD/EFTPC; output.time=time; output.X=X; output.IsJs=IsJs;

while red==1 && it < 60 % [] should be reduced until no EADs appear
    disp('Reducing concentration')
    D=D/step;
    time2=time; X2=X; IsJs2=IsJs;
    parameters.D=D;
    [time, X, IsJs] = runEADprotocol(parameters);
    t=time(find(time==3000,1,'last'):end)-3000; v=X(find(time==3000,1,'last'):end,1);
    d(length(v)) = (v(end) - v(end-1) ) / (t(end)-t(end-1));
    ndx = 2:(length(v)-1);
    d(1)= ( v(2) - v(1) ) / (t(2)-t(1));
    d(ndx) = (v( ndx+1) - v(ndx-1))./ (2 * (t(ndx)-t(ndx-1)));
    [Vp, Vp_index]=max(v);
    ind=find(t>=(t(Vp_index)+150),1);
    d(1:ind)=[];
    EADs = find(d>0.01);
    [Vr, Vr_index]=min(v(ind:end));
    [Vp, Vp_index]=max(v);
    i=i+1;
    disp(['Reducing []. ' num2str(i) ' times'])
    if ~isempty(EADs) || Vr > -40 || Vp < 0
        red=1;
    else
        DEAD=D*step;
        TEAD=DEAD/EFTPC;
        output.time=time2; output.X=X2; output.IsJs=IsJs2;
        red=0;
    end
    it=it+1;
end

disp(['Finished. TEAD = ' num2str(TEAD)])
end
